#ifndef API_CLIENT_H_
#define API_CLIENT_H_

#include <api/config.h>

#include <atomic>
#include <deque>
#include <map>
#include <string>
#include <core/net/http/request.h>
#include <core/net/uri.h>

#include <QJsonDocument>

namespace api {

/**
 * Provide a nice way to access the HTTP API.
 *
 * We don't want our scope's code to be mixed together with HTTP and JSON handling.
 */
class Client {
public:

    ///
    /// \brief The Feed struct
    ///
    struct Feed {
        std::string url;
        std::string title;
        std::string contentSnippet;
        std::string link;
    };
    typedef std::deque<Feed> FeedList;
    struct Feeds {
        FeedList feed;
    };
    ///

    ///
        /// \brief The Feed struct
        ///
        struct Feedout {
            std::string title;
            std::string link;
            std::string publishedDate;
            std::string author;
            std::string content;
        };
        typedef std::deque<Feedout> FeedoutList;
        struct Feedouts {
            FeedoutList feedout;
        };
        ///

    ///
    /// \brief The Load struct
    ///
    struct Omg {
        std::string title;
        std::string link;
        std::string publishedDate;
        std::string author;
        std::string content;
    };
    typedef std::deque<Omg> OmgList;
    struct Omgs {
        OmgList omg;
    };
    ///

    ///
    /// \brief The Load struct
    ///
    struct Load {
        std::string title;
        std::string link;
        std::string publishedDate;
        std::string author;
        std::string content;
    };
    typedef std::deque<Load> LoadList;
    struct Loads {
        LoadList load;
    };
    ///

    ///
    /// \brief The Load struct
    ///
    struct Buntu {
        std::string title;
        std::string link;
        std::string author;
        std::string publishedDate;
        std::string content;
    };
    typedef std::deque<Buntu> BuntuList;
    struct Buntus {
        BuntuList buntu;
    };
    ///

    ///
    struct Plan {
        std::string title;
        std::string link;
        std::string author;
        std::string publishedDate;
        std::string content;
    };
    typedef std::deque<Plan> PlanList;
    struct Plans {
        PlanList plan;
    };
    ///
    ///
    /// \brief The Launch struct
    ///
    struct Launch {
        std::string title;
        std::string link;
        std::string author;
        std::string publishedDate;
        std::string content;
    };
    typedef std::deque<Launch> LaunchList;
    struct Launchs {
        LaunchList launch;
    };
    ///

    ///
    /// \brief The Custom struct
    ///
    struct Custom {
        std::string title;
        std::string link;
        std::string author;
        std::string publishedDate;
        std::string content;
    };
    typedef std::deque<Custom> CustomList;
    struct Customs {
        CustomList custom;
    };
    ///

    /// \brief The Custom struct
    ///
    struct Custom1 {
        std::string title;
        std::string link;
        std::string author;
        std::string publishedDate;
        std::string content;
    };
    typedef std::deque<Custom1> Custom1List;
    struct Custom1s {
        Custom1List custom1;
    };
    ///

    /// \brief The Custom struct
    ///
    struct Custom2 {
        std::string title;
        std::string link;
        std::string author;
        std::string publishedDate;
        std::string content;
    };
    typedef std::deque<Custom2> Custom2List;
    struct Custom2s {
        Custom2List custom2;
    };
    ///


    /// \brief The Custom struct
    ///
    struct Custom3 {
        std::string title;
        std::string link;
        std::string author;
        std::string publishedDate;
        std::string content;
    };
    typedef std::deque<Custom3> Custom3List;
    struct Custom3s {
        Custom3List custom3;
    };
    ///

    /// \brief The Custom struct
    ///
    struct Custom4 {
        std::string title;
        std::string link;
        std::string author;
        std::string publishedDate;
        std::string content;
    };
    typedef std::deque<Custom4> Custom4List;
    struct Custom4s {
        Custom4List custom4;
    };
    ///
    /// \brief Client
    /// \param config
    ///
    Client(Config::Ptr config);

    virtual ~Client() = default;

    /**
     * Get the feed
     */
    virtual Feeds query_feeds(const std::string &query, std::string const& lang_sys);
    virtual Feedouts query_feedouts(const std::string &query, std::string const& lang_sys);

    //ubuntu dept
    virtual Omgs add_omgs(const std::string &add, std::string const& lang_sys);
    virtual Loads add_loads(const std::string &add, std::string const& lang_sys);
    virtual Buntus add_buntus(const std::string &add, std::string const& lang_sys);
    virtual Plans add_plans(const std::string &add, std::string const& lang_sys);
    virtual Launchs add_launchs(const std::string &add, std::string const& lang_sys);
    //custom dept
    virtual Customs add_customs(std::string const& cust_sys, std::string const& lang_sys);
    virtual Custom1s add_custom1s(std::string const& cust1_sys, std::string const& lang_sys);
    virtual Custom2s add_custom2s(std::string const& cust2_sys, std::string const& lang_sys);
    virtual Custom3s add_custom3s(std::string const& cust3_sys, std::string const& lang_sys);
    virtual Custom4s add_custom4s(std::string const& cust4_sys, std::string const& lang_sys);


    /**
     * Cancel any pending queries (this method can be called from a different thread)
     */
    virtual void cancel();

    virtual Config::Ptr config();

protected:
    void get(const core::net::Uri::Path &path,
             const core::net::Uri::QueryParameters &parameters,
             QJsonDocument &root);
    /**
     * Progress callback that allows the query to cancel pending HTTP requests.
     */
    core::net::http::Request::Progress::Next progress_report(
            const core::net::http::Request::Progress& progress);

    /**
     * Hang onto the configuration information
     */
    Config::Ptr config_;

    /**
     * Thread-safe cancelled flag
     */
    std::atomic<bool> cancelled_;
};

}

#endif // API_CLIENT_H_

