#include <api/client.h>

#include <core/net/error.h>
#include <core/net/http/client.h>
#include <core/net/http/content_type.h>
#include <core/net/http/response.h>
#include <QVariantMap>
#include <iostream>

namespace http = core::net::http;
namespace net = core::net;

using namespace api;
using namespace std;

Client::Client(Config::Ptr config) :
    config_(config), cancelled_(false) {
}


void Client::get(const net::Uri::Path &path,
                 const net::Uri::QueryParameters &parameters, QJsonDocument &root) {
    // Create a new HTTP client
    auto client = http::make_client();

    // Start building the request configuration
    http::Request::Configuration configuration;

    // Build the URI from its components
    net::Uri uri = net::make_uri(config_->apiroot, path, parameters);
    configuration.uri = client->uri_to_string(uri);

    // Give out a user agent string
    configuration.header.add("User-Agent", config_->user_agent);

    // Build a HTTP request object from our configuration
    auto request = client->head(configuration);

    try {
        // Synchronously make the HTTP request
        // We bind the cancellable callback to #progress_report
        auto response = request->execute(
                    bind(&Client::progress_report, this, placeholders::_1));

        // Check that we got a sensible HTTP status code
        if (response.status != http::Status::ok) {
            throw domain_error(response.body);
        }
        // Parse the JSON from the response
        root = QJsonDocument::fromJson(response.body.c_str());



    } catch (net::Error &) {
    }
}

///
/// \brief Client::query_feeds
/// \param query
/// \param lang_sys
/// \return
///


Client::Feeds Client::query_feeds(const string& query, std::string const& lang_sys) {
    QJsonDocument root;


    // Build a URI and get the contents
    // The fist parameter forms the path part of the URI.
    // The second parameter forms the CGI parameters.
    get( { "feed", "find" }, {{ "q", query }, {"v", "1.0"}, { "hl", lang_sys }}, root);
    // e.g. https://ajax.googleapis.com/ajax/services/feed/find?q=Ubuntu&v=1.0

    Feeds result;

    QVariantMap variant = root.toVariant().toMap();


    QVariantMap responseData = variant["responseData"].toMap();
    // Iterate
    for (const QVariant &i : responseData["entries"].toList()) {
        QVariantMap item = i.toMap();
        // Add a result to the list
        result.feed.emplace_back(
                    Feed { item["title"].toString().toStdString(),
                            item["contentSnippet"].toString().toStdString(),
                            item["url"].toString().toStdString(),
                            item["link"].toString().toStdString(),
                    });

    }
    return result;
}
///
///
/// \brief Client::query_feedouts
/// \param query
/// \param lang_sys
/// \return
///
Client::Feedouts Client::query_feedouts(const string& linky, std::string const& lang_sys) {
    QJsonDocument root;


        get( { "feed", "load" }, {{ "q", linky }, {"v", "1.0"}, { "hl", lang_sys }}, root);
        // e.g. https://ajax.googleapis.com/ajax/services/feed/load?q=Ubuntu&v=1.0
        ///
        /// \brief result
        ///
        Feedouts result;
        QVariantMap variant = root.toVariant().toMap();


        QVariantMap responseData = variant["responseData"].toMap();

        QVariantMap feed = responseData["feed"].toMap();
        // Iterate through the data
        for (const QVariant &i : feed["entries"].toList()) {
            QVariantMap item = i.toMap();
            // Add a result to the list
            result.feedout.emplace_back(
                        Feedout { item["title"].toString().toStdString(),
                                item["link"].toString().toStdString(),
                                item["publishedDate"].toString().toStdString(),
                                item["author"].toString().toStdString(),
                                item["content"].toString().toStdString(),
                        });


   }



    return result;
}
///

///
/// \brief Client::add_loads
/// \param add
/// \param lang_sys
/// \return
///
Client::Omgs Client::add_omgs(const string& add, std::string const& lang_sys) {
    QJsonDocument root;

    // Build a URI and get the contents
    // The fist parameter forms the path part of the URI.
    // The second parameter forms the CGI parameters.
    get( { "feed", "load" }, {{ "q", add }, {"v", "1.0"}, { "hl", lang_sys }}, root);
    // e.g. https://ajax.googleapis.com/ajax/services/feed/load?q=Ubuntu&v=1.0

    Omgs result;

    QVariantMap variant = root.toVariant().toMap();


    QVariantMap responseData = variant["responseData"].toMap();

    QVariantMap feed = responseData["feed"].toMap();
    // Iterate through the data
    for (const QVariant &i : feed["entries"].toList()) {
        QVariantMap item = i.toMap();
        // Add a result to the list
        result.omg.emplace_back(
                    Omg { item["title"].toString().toStdString(),
                            item["link"].toString().toStdString(),
                            item["publishedDate"].toString().toStdString(),
                            item["author"].toString().toStdString(),
                            item["content"].toString().toStdString(),
                    });
    }

    return result;
}
///


///
/// \brief Client::add_loads
/// \param add
/// \param lang_sys
/// \return
///
Client::Loads Client::add_loads(const string& add, std::string const& lang_sys) {
    QJsonDocument root;

    // Build a URI and get the contents
    // The fist parameter forms the path part of the URI.
    // The second parameter forms the CGI parameters.
    get( { "feed", "load" }, {{ "q", add }, {"v", "1.0"}, { "hl", lang_sys }}, root);
    // e.g. https://ajax.googleapis.com/ajax/services/feed/load?q=Ubuntu&v=1.0

    Loads result;

    QVariantMap variant = root.toVariant().toMap();


    QVariantMap responseData = variant["responseData"].toMap();

    QVariantMap feed = responseData["feed"].toMap();
    // Iterate through the data
    for (const QVariant &i : feed["entries"].toList()) {
        QVariantMap item = i.toMap();
        // Add a result to the list
        result.load.emplace_back(
                    Load { item["title"].toString().toStdString(),
                            item["link"].toString().toStdString(),
                            item["publishedDate"].toString().toStdString(),
                            item["author"].toString().toStdString(),
                            item["content"].toString().toStdString()
                    });
    }

    return result;
}
///

///
/// \brief Client::add_buntus
/// \param add
/// \param lang_sys
/// \return
///
Client::Buntus Client::add_buntus(const string& add, std::string const& lang_sys) {
    QJsonDocument root;

    // Build a URI and get the contents
    // The fist parameter forms the path part of the URI.
    // The second parameter forms the CGI parameters.
    get( { "feed", "load" }, {{ "q", add }, {"v", "1.0"}, { "hl", lang_sys }}, root);
    // e.g. https://ajax.googleapis.com/ajax/services/feed/load?q=Ubuntu&v=1.0

    Buntus result;

    QVariantMap variant = root.toVariant().toMap();


    QVariantMap responseData = variant["responseData"].toMap();

    QVariantMap feed = responseData["feed"].toMap();
    // Iterate through the data
    for (const QVariant &i : feed["entries"].toList()) {
        QVariantMap item = i.toMap();
        // Add a result to the list
        result.buntu.emplace_back(
                    Buntu { item["title"].toString().toStdString(),
                            item["link"].toString().toStdString(),
                            item["author"].toString().toStdString(),
                            item["publishedDate"].toString().toStdString(),
                            item["content"].toString().toStdString(),
                    });
    }

    return result;
}
///

///
/// \brief Client::add_launchs
/// \param add
/// \param lang_sys
/// \return
///
Client::Plans Client::add_plans(const string& add, std::string const& lang_sys) {
    QJsonDocument root;

    // Build a URI and get the contents
    // The fist parameter forms the path part of the URI.
    // The second parameter forms the CGI parameters.
    get( { "feed", "load" }, {{ "q", add }, {"v", "1.0"}, { "hl", lang_sys }}, root);
    // e.g. https://ajax.googleapis.com/ajax/services/feed/load?q=Ubuntu&v=1.0

    Plans result;

    QVariantMap variant = root.toVariant().toMap();


    QVariantMap responseData = variant["responseData"].toMap();

    QVariantMap feed = responseData["feed"].toMap();
    // Iterate through the data
    for (const QVariant &i : feed["entries"].toList()) {
        QVariantMap item = i.toMap();
        // Add a result to the list
        result.plan.emplace_back(
                    Plan { item["title"].toString().toStdString(),
                            item["link"].toString().toStdString(),
                            item["author"].toString().toStdString(),
                            item["publishedDate"].toString().toStdString(),
                            item["content"].toString().toStdString(),
                    });
    }

    return result;
}
///

///
/// \brief Client::add_launchs
/// \param add
/// \param lang_sys
/// \return
///
Client::Launchs Client::add_launchs(const string& add, std::string const& lang_sys) {
    QJsonDocument root;

    // Build a URI and get the contents
    // The fist parameter forms the path part of the URI.
    // The second parameter forms the CGI parameters.
    get( { "feed", "load" }, {{ "q", add }, {"v", "1.0"}, { "hl", lang_sys }}, root);
    // e.g. https://ajax.googleapis.com/ajax/services/feed/load?q=Ubuntu&v=1.0

    Launchs result;

    QVariantMap variant = root.toVariant().toMap();


    QVariantMap responseData = variant["responseData"].toMap();

    QVariantMap feed = responseData["feed"].toMap();
    // Iterate through the data
    for (const QVariant &i : feed["entries"].toList()) {
        QVariantMap item = i.toMap();
        // Add a result to the list
        result.launch.emplace_back(
                    Launch { item["title"].toString().toStdString(),
                            item["link"].toString().toStdString(),
                            item["author"].toString().toStdString(),
                            item["publishedDate"].toString().toStdString(),
                            item["content"].toString().toStdString(),
                    });
    }

    return result;
}
///

///
/// \brief Client::add_customs
/// \param cust_sys
/// \param lang_sys
/// \return
///
Client::Customs Client::add_customs(std::string const& cust_sys, std::string const& lang_sys) {
    QJsonDocument root;

    // Build a URI and get the contents
    // The fist parameter forms the path part of the URI.
    // The second parameter forms the CGI parameters.
    get( { "feed", "load" }, {{ "q", "http://" + cust_sys }, {"v", "1.0"}, { "hl", lang_sys }}, root);
    // e.g. https://ajax.googleapis.com/ajax/services/feed/load?q=Ubuntu&v=1.0

    Customs result;

    QVariantMap variant = root.toVariant().toMap();


     QVariantMap responseData = variant["responseData"].toMap();

     QVariantMap feed = responseData["feed"].toMap();
    // Iterate through the  data
    for (const QVariant &i : feed["entries"].toList()) {
        QVariantMap item = i.toMap();
       // QVariantMap images = item["image"].toMap();
        // Add a result to the list
        result.custom.emplace_back(
                    Custom { item["title"].toString().toStdString(),
                            item["link"].toString().toStdString(),
                            item["author"].toString().toStdString(),
                            item["publishedDate"].toString().toStdString(),
                            item["content"].toString().toStdString(),
                    });
    }

    return result;
}
///

///
/// \brief Client::add_customs
/// \param cust_sys
/// \param lang_sys
/// \return
///
Client::Custom1s Client::add_custom1s(std::string const& cust_sys1, std::string const& lang_sys) {
    QJsonDocument root;

    // Build a URI and get the contents
    // The fist parameter forms the path part of the URI.
    // The second parameter forms the CGI parameters.
    get( { "feed", "load" }, {{ "q", "http://" + cust_sys1 }, {"v", "1.0"}, { "hl", lang_sys }}, root);
    // e.g. https://ajax.googleapis.com/ajax/services/feed/load?q=Ubuntu&v=1.0

    Custom1s result;

    QVariantMap variant = root.toVariant().toMap();


     QVariantMap responseData = variant["responseData"].toMap();

     QVariantMap feed = responseData["feed"].toMap();
    // Iterate through the  data
    for (const QVariant &i : feed["entries"].toList()) {
        QVariantMap item = i.toMap();
       // QVariantMap images = item["image"].toMap();
        // Add a result to the list
        result.custom1.emplace_back(
                    Custom1 { item["title"].toString().toStdString(),
                            item["link"].toString().toStdString(),
                            item["author"].toString().toStdString(),
                            item["publishedDate"].toString().toStdString(),
                            item["content"].toString().toStdString(),
                    });
    }

    return result;
}
///
///

///
/// \brief Client::add_customs
/// \param cust_sys
/// \param lang_sys
/// \return
///
Client::Custom2s Client::add_custom2s(std::string const& cust_sys2, std::string const& lang_sys) {
    QJsonDocument root;

    // Build a URI and get the contents
    // The fist parameter forms the path part of the URI.
    // The second parameter forms the CGI parameters.
    get( { "feed", "load" }, {{ "q", "http://" + cust_sys2 }, {"v", "1.0"}, { "hl", lang_sys }}, root);
    // e.g. https://ajax.googleapis.com/ajax/services/feed/load?q=Ubuntu&v=1.0

    Custom2s result;

    QVariantMap variant = root.toVariant().toMap();


     QVariantMap responseData = variant["responseData"].toMap();

     QVariantMap feed = responseData["feed"].toMap();
    // Iterate through the  data
    for (const QVariant &i : feed["entries"].toList()) {
        QVariantMap item = i.toMap();
       // QVariantMap images = item["image"].toMap();
        // Add a result to the list
        result.custom2.emplace_back(
                    Custom2 { item["title"].toString().toStdString(),
                            item["link"].toString().toStdString(),
                            item["author"].toString().toStdString(),
                            item["publishedDate"].toString().toStdString(),
                            item["content"].toString().toStdString(),
                    });
    }

    return result;
}
///

///
/// \brief Client::add_customs
/// \param cust_sys
/// \param lang_sys
/// \return
///
Client::Custom3s Client::add_custom3s(std::string const& cust_sys3, std::string const& lang_sys) {
    QJsonDocument root;

    // Build a URI and get the contents
    // The fist parameter forms the path part of the URI.
    // The second parameter forms the CGI parameters.
    get( { "feed", "load" }, {{ "q", "http://" + cust_sys3 }, {"v", "1.0"}, { "hl", lang_sys }}, root);
    // e.g. https://ajax.googleapis.com/ajax/services/feed/load?q=Ubuntu&v=1.0

    Custom3s result;

    QVariantMap variant = root.toVariant().toMap();


     QVariantMap responseData = variant["responseData"].toMap();

     QVariantMap feed = responseData["feed"].toMap();
    // Iterate through the  data
    for (const QVariant &i : feed["entries"].toList()) {
        QVariantMap item = i.toMap();
       // QVariantMap images = item["image"].toMap();
        // Add a result to the list
        result.custom3.emplace_back(
                    Custom3 { item["title"].toString().toStdString(),
                            item["link"].toString().toStdString(),
                            item["author"].toString().toStdString(),
                            item["publishedDate"].toString().toStdString(),
                            item["content"].toString().toStdString(),
                    });
    }

    return result;
}
///

///
/// \brief Client::add_customs
/// \param cust_sys
/// \param lang_sys
/// \return
///
Client::Custom4s Client::add_custom4s(std::string const& cust_sys4, std::string const& lang_sys) {
    QJsonDocument root;

    // Build a URI and get the contents
    // The fist parameter forms the path part of the URI.
    // The second parameter forms the CGI parameters.
    get( { "feed", "load" }, {{ "q", "http://" + cust_sys4 }, {"v", "1.0"}, { "hl", lang_sys }}, root);
    // e.g. https://ajax.googleapis.com/ajax/services/feed/load?q=Ubuntu&v=1.0

    Custom4s result;

    QVariantMap variant = root.toVariant().toMap();


     QVariantMap responseData = variant["responseData"].toMap();

     QVariantMap feed = responseData["feed"].toMap();
    // Iterate through the  data
    for (const QVariant &i : feed["entries"].toList()) {
        QVariantMap item = i.toMap();
       // QVariantMap images = item["image"].toMap();
        // Add a result to the list
        result.custom4.emplace_back(
                    Custom4 { item["title"].toString().toStdString(),
                            item["link"].toString().toStdString(),
                            item["author"].toString().toStdString(),
                            item["publishedDate"].toString().toStdString(),
                            item["content"].toString().toStdString(),
                    });
    }

    return result;
}
///

/// \brief Client::progress_report
/// \return
///
http::Request::Progress::Next Client::progress_report(
        const http::Request::Progress&) {

    return cancelled_ ?
                http::Request::Progress::Next::abort_operation :
                http::Request::Progress::Next::continue_operation;
}

void Client::cancel() {
    cancelled_ = true;
}

Config::Ptr Client::config() {
    return config_;
}

