#include <boost/algorithm/string/trim.hpp>

#include <scope/localization.h>
#include <scope/query.h>

#include <unity/scopes/Annotation.h>
#include <unity/scopes/CategorisedResult.h>
#include <unity/scopes/CategoryRenderer.h>
#include <unity/scopes/QueryBase.h>
#include <unity/scopes/SearchReply.h>

#include <iomanip>
#include <sstream>

namespace sc = unity::scopes;
namespace alg = boost::algorithm;

using namespace std;
using namespace api;
using namespace scope;


/**
 * Define the layout for the forecast results
 *
 * The icon size is small, and ask for the card layout
 * itself to be horizontal. I.e. the text will be placed
 * next to the image.
 */
const static string FEED_TEMPLATE =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "vertical-journal",
        "card-layout": "horizontal",
        "card-size" : "small"
        },
        "components": {
        "title": "title",
        "overlay" : "true",
        "subtitle": "subtitle",
        "emblem" : "art"
        }
        }
        )";
const static string LOAD_TEMPLATE =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "vertical-journal",
        "card-layout": "horizontal"
        },
        "components": {
        "title": "title",
        "emblem" : "art",
        "subtitle": "subtitle"
        }
        }
        )";
const static string BUNTU_TEMPLATE =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "vertical-journal",
        "card-layout": "horizontal"
        },
        "components": {
        "title": "title",
        "emblem" : "art",
        "subtitle": "subtitle"
        }
        }
        )";
const static string LAUNCH_TEMPLATE =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "vertical-journal",
        "card-layout": "horizontal"
        },
        "components": {
        "title": "title",
        "emblem" : "art",
        "subtitle": "subtitle"
        }
        }
        )";
 const static string CUSTOM_TEMPLATE =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "vertical-journal",
        "card-layout": "horizontal"
        },
        "components": {
        "title": "title",
        "emblem" : "art",
        "subtitle": "subtitle"
        }
        }
        )";

Query::Query(const sc::CannedQuery &query, const sc::SearchMetadata &metadata,
             Config::Ptr config) :
    sc::SearchQueryBase(query, metadata), client_(config) {
}

void Query::cancelled() {
    client_.cancel();
}




void Query::run(sc::SearchReplyProxy const& reply) {
    try {

        // Create the root department
        sc::Department::SPtr all_depts = sc::Department::create("", query(), "Discovery");
        // Create two other departments
        sc::Department::SPtr custom_department = sc::Department::create("Custom", query(), "Custom");
        sc::Department::SPtr ubuntu_department = sc::Department::create("Ubuntu", query(), "Ubuntu");
        // Register these as subdepartments of the root
        all_depts->set_subdepartments({custom_department,
                                       ubuntu_department});
        // Register departments on the reply
        reply->register_departments(all_depts);


        // Start by getting information about the query
        const sc::CannedQuery &query(sc::SearchQueryBase::query());

        // Trim the query string of whitespace
        string query_string = alg::trim_copy(query.query_string());

        //dept stuff
        //const bool empty_query = query_string.size() == 0;
        const string department = query.department_id();
        //

        // LANG Setting
        auto lang_sys = settings().at("lang").get_string();
        //

        // cust settings
        auto cust_sys = settings().at("cust").get_string();
        auto cust1_sys = settings().at("cust1").get_string();
        auto cust2_sys = settings().at("cust2").get_string();
        auto cust3_sys = settings().at("cust3").get_string();
        auto cust4_sys = settings().at("cust4").get_string();


        //// hide some stuff in default

        ///
        /// \brief feeds
        ///
        Client::Feeds feeds;

        //if (query_string.empty()) { //we don't really want it empty on feed finding...
        if (department == "") { ///DEPARTMENT

        if (!query.department_id().empty()) {
            // If a department is selected, get the department info
            feeds = client_.query_feeds(query.department_id(), lang_sys);
        } else if (query_string.empty()) {
            // If there is no search string, get the ubuntu feeds
            feeds = client_.query_feeds("Ubuntu", lang_sys);
        //}
        } else {
            // otherwise, get the forecast for the search string
            feeds = client_.query_feeds(query_string, lang_sys);
        }
        }///DEPARTMENT

        // Register a category for finding feed
        auto feed_cat = reply->register_category("Find",
                                                     "Find Feeds", "", sc::CategoryRenderer(FEED_TEMPLATE));

        // For each of the feeds
        for (const auto &feed : feeds.feed) {
            // Create a result
            sc::CategorisedResult res(feed_cat);

            // We must have a URI
            res.set_uri(feed.link);

            res.set_title(feed.title); //url

            // Set the rest of the attributes
            res.set_art("http://www.google.com/s2/favicons?domain=" + feed.link);
            //res.set_art(feed.link + "apple-touch-icon.png");

            res["subtitle"] = feed.contentSnippet;
            res["description"] = feed.link;

            // Push the result
            if (!reply->push(res)) {
                // If we fail to push, it means the query has been cancelled.
                // So don't continue;
                return;
            }
        }
        ///




        ///
       if (department == "Ubuntu") { ///DEPARTMENT
               ///OMGS
               Client::Omgs omgs;



               // If there is no search string, get the
               omgs = client_.add_omgs("http://www.omgubuntu.co.uk/feed", lang_sys);
               // Register a category for the
               auto omg_cat = reply->register_category("Omg",
                                                            _("OMG! Ubuntu!"), "", sc::CategoryRenderer(LAUNCH_TEMPLATE));
               // For each of the forecast days
               for (const auto &omg : omgs.omg) {
                   // Create a result
                   sc::CategorisedResult res(omg_cat);

                   // We must have a URI
                   res.set_uri(omg.link);

                   res.set_title(omg.title);
                   //res.set_art(omg.icon);
                   // Set the rest of the attributes
                   res.set_art("http://www.google.com/s2/favicons?domain=" + omg.link);
                   res["subtitle"] = omg.author;
                   res["date"] = omg.publishedDate;
                   res["description"] = omg.content;
                   // Push the result
                   if (!reply->push(res)) {
                       // If we fail to push, it means the query has been cancelled.
                       // So don't continue;
                       return;
                   }
                   }
                   }
               ///OMGS


       auto add_value = ("http://www.ubuntu.com/usn/rss.xml");

if (department == "Ubuntu") { ///DEPARTMENT
        ///LOADS
        ///
        Client::Loads loads;

        ///
            // If a department is selected, get the department info
            loads = client_.add_loads(add_value, lang_sys);
            //myname = _("Ubuntu");


        auto load_cat = reply->register_category("Load",
                                                     _("Ubuntu Security Notices"), "", sc::CategoryRenderer(LOAD_TEMPLATE));
        // If there is no search string
       /// loads = client_.add_loads(add_value, lang_sys);
        // Register a category

        // For each of...
        for (const auto &load : loads.load) {
            // Create a result
            sc::CategorisedResult res(load_cat);

            // We must have a URI
            res.set_uri(load.link);

            res.set_title(load.title);
            //res.set_art(load.icon);
            // Set the rest of the attributes
            res.set_art("http://www.google.com/s2/favicons?domain=" + load.link);
            res["subtitle"] = load.publishedDate;
            res["description"] = load.content;
            // Push the result
            if (!reply->push(res)) {
                // If we fail to push, it means the query has been cancelled.
                // So don't continue;
                return;

            }}
            }
        ///LOADS


        ///
        /// \brief buntus
        ///
        Client::Buntus buntus;

        if (department == "Ubuntu") { ///DEPARTMENT
            // If there is no search string
        buntus = client_.add_buntus("http://2buntu.com/rss/", lang_sys);


        // Register a category
        auto buntu_cat = reply->register_category("Buntu",
                                                     _("2buntu"), "", sc::CategoryRenderer(BUNTU_TEMPLATE));


        // For each of the days
        for (const auto &buntu : buntus.buntu) {
            // Create a result
            sc::CategorisedResult res(buntu_cat);

            // We must have a URI
            res.set_uri(buntu.link);

            res.set_title(buntu.title);
            //res.set_art(load.icon);
            // Set the rest of the attributes
            res.set_art("http://www.google.com/s2/favicons?domain=" + buntu.link);
            res["subtitle"] = buntu.author;
            //res["attributes"] = buntu.publishedDate;
            res["description"] =  buntu.content;


            // Push the result
            if (!reply->push(res)) {
                // If we fail to push, it means the query has been cancelled.
                // So don't continue;
                return;
            }
        }
        }
        /// Buntu
        ///

        ///
               if (department == "Ubuntu") { ///DEPARTMENT
                       ///PLAN
                       Client::Plans plans;



                       // If there is no search string, get the
                       plans = client_.add_plans("http://planet.ubuntu.com/rss20.xml", lang_sys);
                       // Register a category for the
                       auto plan_cat = reply->register_category("Plan",
                                                                    _("Planet Ubuntu"), "", sc::CategoryRenderer(LAUNCH_TEMPLATE));
                       // For each of the forecast days
                       for (const auto &plan : plans.plan) {
                           // Create a result
                           sc::CategorisedResult res(plan_cat);

                           // We must have a URI
                           res.set_uri(plan.link);

                           res.set_title(plan.title);
                           //res.set_art(plan.icon);
                           // Set the rest of the attributes
                           res.set_art("http://www.google.com/s2/favicons?domain=" + plan.link);
                           res["subtitle"] = plan.publishedDate;
                           res["description"] = plan.content;
                           // Push the result
                           if (!reply->push(res)) {
                               // If we fail to push, it means the query has been cancelled.
                               // So don't continue;
                               return;
                           }
                           }
                           }
                       ///PLAN

        ///
       if (department == "Ubuntu") { ///DEPARTMENT
               ///LAUNCHS
               Client::Launchs launchs;

               // If there is no search string...
               launchs = client_.add_launchs("http://feeds.launchpad.net/ubuntu/announcements.atom", lang_sys);
               // Register a category for the
               auto launch_cat = reply->register_category("Launch",
                                                            _("Launchpad Ubuntu Announcements"), "", sc::CategoryRenderer(LAUNCH_TEMPLATE));
               // For each of the...
               for (const auto &launch : launchs.launch) {
                   // Create a result
                   sc::CategorisedResult res(launch_cat);

                   // We must have a URI
                   res.set_uri(launch.link);

                   res.set_title(launch.title);
                   //res.set_art(launch.icon);
                   // Set the rest of the attributes
                   res.set_art("http://www.google.com/s2/favicons?domain=" + launch.link);
                   res["subtitle"] = launch.publishedDate;
                   res["description"] = launch.content;
                   // Push the result
                   if (!reply->push(res)) {
                       // If we fail to push, it means the query has been cancelled.
                       // So don't continue;
                       return;
                   }
                   }
                   }///DEPART
               ///LAUNCHS


        ///CUSTOM
        Client::Customs customs;

        if (department == "Custom") { ///DEPARTMENT

            // If there is no search string
            customs = client_.add_customs(cust_sys, lang_sys);


        // Register a category
        auto custom_cat = reply->register_category("Custom",
                                                     cust_sys, "", sc::CategoryRenderer(CUSTOM_TEMPLATE));

        // For each of the...
        for (const auto &custom : customs.custom) {
            // Create a result
            sc::CategorisedResult res(custom_cat);

            // We must have a URI
            res.set_uri(custom.link);

            res.set_title(custom.title);
            //res.set_art(custom.icon);
            // Set the rest of the attributes
            res.set_art("http://www.google.com/s2/favicons?domain=" + custom.link);
            res["subtitle"] = custom.publishedDate;
            res["description"] = custom.content;

            // Push the result
            if (!reply->push(res)) {
                // If we fail to push, it means the query has been cancelled.
                // So don't continue;
                return;
            }
        }
        }
        ///CUSTOM
        ///
        ///CUSTOM1
              Client::Custom1s custom1s;

              if (department == "Custom") { ///DEPARTMENT

                  // If there is no search string
                  custom1s = client_.add_custom1s(cust1_sys, lang_sys);


              // Register a category
              auto custom1_cat = reply->register_category("Custom1",
                                                           cust1_sys, "", sc::CategoryRenderer(CUSTOM_TEMPLATE));

              // For each of the...
              for (const auto &custom1 : custom1s.custom1) {
                  // Create a result
                  sc::CategorisedResult res(custom1_cat);

                  // We must have a URI
                  res.set_uri(custom1.link);

                  res.set_title(custom1.title);
                  //res.set_art(custom.icon);
                  // Set the rest of the attributes
                  res.set_art("http://www.google.com/s2/favicons?domain=" + custom1.link);
                  res["subtitle"] = custom1.publishedDate;
                  res["description"] = custom1.content;

                  // Push the result
                  if (!reply->push(res)) {
                      // If we fail to push, it means the query has been cancelled.
                      // So don't continue;
                      return;
                  }
              }
              }
              ///CUSTOM1
              ///

              ///CUSTOM2
                    Client::Custom2s custom2s;

                    if (department == "Custom") { ///DEPARTMENT

                        // If there is no search string
                        custom2s = client_.add_custom2s(cust2_sys, lang_sys);


                    // Register a category
                    auto custom2_cat = reply->register_category("Custom2",
                                                                 cust2_sys, "", sc::CategoryRenderer(CUSTOM_TEMPLATE));

                    // For each of the...
                    for (const auto &custom2 : custom2s.custom2) {
                        // Create a result
                        sc::CategorisedResult res(custom2_cat);

                        // We must have a URI
                        res.set_uri(custom2.link);

                        res.set_title(custom2.title);
                        //res.set_art(custom.icon);
                        // Set the rest of the attributes
                        res.set_art("http://www.google.com/s2/favicons?domain=" + custom2.link);
                        res["subtitle"] = custom2.publishedDate;
                        res["description"] = custom2.content;

                        // Push the result
                        if (!reply->push(res)) {
                            // If we fail to push, it means the query has been cancelled.
                            // So don't continue;
                            return;
                        }
                    }
                    }
                    ///CUSTOM2
                    ///

                    ///CUSTOM3
                          Client::Custom3s custom3s;

                          if (department == "Custom") { ///DEPARTMENT

                              // If there is no search string
                              custom3s = client_.add_custom3s(cust3_sys, lang_sys);


                          // Register a category
                          auto custom3_cat = reply->register_category("Custom3",
                                                                       cust3_sys, "", sc::CategoryRenderer(CUSTOM_TEMPLATE));

                          // For each of the...
                          for (const auto &custom3 : custom3s.custom3) {
                              // Create a result
                              sc::CategorisedResult res(custom3_cat);

                              // We must have a URI
                              res.set_uri(custom3.link);

                              res.set_title(custom3.title);
                              //res.set_art(custom.icon);
                              // Set the rest of the attributes
                              res.set_art("http://www.google.com/s2/favicons?domain=" + custom3.link);
                              res["subtitle"] = custom3.publishedDate;
                              res["description"] = custom3.content;

                              // Push the result
                              if (!reply->push(res)) {
                                  // If we fail to push, it means the query has been cancelled.
                                  // So don't continue;
                                  return;
                              }
                          }
                          }
                          ///CUSTOM3
                          ///

                          ///CUSTOM4
                                Client::Custom4s custom4s;

                                if (department == "Custom") { ///DEPARTMENT

                                    // If there is no search string
                                    custom4s = client_.add_custom4s(cust4_sys, lang_sys);


                                // Register a category
                                auto custom4_cat = reply->register_category("Custom4",
                                                                             cust4_sys, "", sc::CategoryRenderer(CUSTOM_TEMPLATE));

                                // For each of the...
                                for (const auto &custom4 : custom4s.custom4) {
                                    // Create a result
                                    sc::CategorisedResult res(custom4_cat);

                                    // We must have a URI
                                    res.set_uri(custom4.link);

                                    res.set_title(custom4.title);
                                    //res.set_art(custom.icon);
                                    // Set the rest of the attributes
                                    res.set_art("http://www.google.com/s2/favicons?domain=" + custom4.link);
                                    res["subtitle"] = custom4.publishedDate;
                                    res["description"] = custom4.content;

                                    // Push the result
                                    if (!reply->push(res)) {
                                        // If we fail to push, it means the query has been cancelled.
                                        // So don't continue;
                                        return;
                                    }
                                }
                                }
                                ///CUSTOM4
                                ///

    } catch (domain_error &e) {
        // Handle exceptions being thrown by the client API
        cerr << e.what() << endl;
        reply->error(current_exception());
    }
}

